function [dstarout,c1out,c2out,tcrout] = dstar(x1,x2,x3,method,plotsurf)

%
% dstar: estimate and construct confidence intervals for D*, c1, c2, and
%        provides point estimates for the TCRs
%
% usage: [dstarout,c1out,c2out,tcrout] = dstar(x1,x2,x3,method,plotsurf)
%
%
%=============================================================================%
%
% arguments: (input)
%  x1  - Biomarker scores for group 1. Must be row or column vector.
%  x2  - Biomarker scores for group 2. Must be row or column vector.
%  x3  - Biomarker scores for group 3. Must be row or column vector.
% 
%  method - Method for analysis. Includes the following:
%             - 'normal' - assumes x1,x2,x3 are each normally distributed.
%             - 'boxcox' - assumes x1,x2,x3 can be transformed to normality.
%             - 'kernel' - nonparametric approach based on normal kernels.
%
%  plotsurf - Plots the ROC surface. Accepts 'yes' or 'none'. If left
%             empty, defaults to 'none'.
%
%=============================================================================%
%
% arguments: (output)
%  dstarout - vector containing:
%             [(point estimate of D*) (D* CI lower limit) (D* CI upper limit)]
%
%  c1out    - vector containing:
%             [(point estimate of c1) (c1 CI lower limit) (c1 CI upper limit)]
%
%  c2out    - vector containing:
%             [(point estimate of c2) (c2 CI lower limit) (c2 CI upper limit)]
%
%  tcrout   - vector containing point estimates of:
%             [tcr1 tcr2 tcr3]
%
%=============================================================================%

%=========================================================%
% Begin Error Checking:
%=========================================================%

if nargin<3; error('The function ''dstar'' requires at least 3 arguments');end
if ~exist('plotsurf','var')
    plotsurf = 'none';
end
if nargin>5; error('Too many input arguments in function ''dstar'' ');end
if isnumeric(x1)~=1;error('Biomarker scores for group 1 must be numeric');end
if isnumeric(x2)~=1;error('Biomarker scores for group 2 must be numeric');end
if isnumeric(x3)~=1;error('Biomarker scores for group 3 must be numeric');end

sizex1=size(x1);
sizex2=size(x2);
sizex3=size(x3);

if sizex1(1)>1 && sizex1(2)>1;error('Input argument ''x1'' must be a row or a column vector');end
if sizex2(1)>1 && sizex2(2)>1;error('Input argument ''x2'' must be a row or a column vector');end
if sizex3(1)>1 && sizex3(2)>1;error('Input argument ''x3'' must be a row or a column vector');end

listm={'normal','boxcox','kernel'};
wm=sum(strncmp(method,listm,6));

if wm==0;error('Invalid value of ''function'' parameter. You must provide one of the following: ''normal'' or ''boxcox'' or ''kernel'' ');end

if nargin==5
        list={'yes','none'};
        w=sum(strncmp(plotsurf,list,4));
        if w==0;error('Invalid value of ''function'' parameter. You must provide one of the following: ''yes'' or ''none'' ');end
end



warning('off','all')

%=========================================================%
% Assuming normality -  use delta method                  %
%=========================================================%

rng(1234,'twister')

if method == 'normal'
    xbar1 = mean(x1);
    xbar2 = mean(x2);
    xbar3 = mean(x3);
    sd1 = std(x1)*sqrt((length(x1)-1)/length(x1));
    sd2 = std(x2)*sqrt((length(x2)-1)/length(x2));
    sd3 = std(x3)*sqrt((length(x3)-1)/length(x3));
    
    %sd1 = std(x1);
    %sd2 = std(x3);
    %sd3 = std(x3);
    
    dist = @(c)(sqrt((1 - normcdf(c(1),xbar1,sd1))^2 + (1 - (normcdf(c(2),xbar2,sd2)...
                - normcdf(c(1),xbar2,sd2)))^2 + ...
               (normcdf(c(2),xbar3,sd3))^2)); %function to be minimized

    c0 = [(xbar1+xbar2)/2 (xbar2+xbar3)/2];
    options = optimset('MaxIter',20000,'MaxFunEvals',20000,'TolX',10e-15);
    [cstar, dstar]= fminsearch(dist,c0,options);
    c1dstar= cstar(1);
    c2dstar= cstar(2);

    
    varmu1 = sd1^2/length(x1);
    varsd1 = sd1^2/(2*(length(x1)-1));
    varmu2 = sd2^2/length(x2);
    varsd2 = sd2^2/(2*(length(x2)-1));
    varmu3 = sd3^2/length(x3);
    varsd3 = sd3^2/(2*(length(x3)-1));

    diagval = [varmu1 varsd1 varmu2 varsd2 varmu3 varsd3];
    V = diag(diagval);
    
    TCR1ds = normcdf((c1dstar-xbar1)/sd1);
    TCR2ds = normcdf((c2dstar-xbar2)/sd2) - normcdf((c1dstar-xbar2)/sd2);
    TCR3ds = normcdf((xbar3-c2dstar)/sd3);
    
    % Delta method for D*
    n1 = length(x1);
    n2 = length(x2);
    n3 = length(x3);
    varmu1 = sd1^2/n1;
    varsd1 = sd1^2/(2*(n1-1));
    varmu2 = sd2^2/n2;
    varsd2 = sd2^2/(2*(n2-1));
    varmu3 = sd3^2/n3;
    varsd3 = sd3^2/(2*(n3-1));

    diagval = [varmu1 varsd1 varmu2 varsd2 varmu3 varsd3];
    V = diag(diagval);
    
    ddstarmu1 = derivest(@(xbar)dstarfunmu1(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar1);
    ddstarmu2 = derivest(@(xbar)dstarfunmu2(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar2);
    ddstarmu3 = derivest(@(xbar)dstarfunmu3(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar3);

    ddstarsd1 = derivest(@(sd)dstarfunsd1(sd,xbar1,xbar2,sd2,xbar3,sd3),sd1);
    ddstarsd2 = derivest(@(sd)dstarfunsd2(sd,xbar1,sd1,xbar2,xbar3,sd3),sd2);
    ddstarsd3 = derivest(@(sd)dstarfunsd3(sd,xbar1,sd1,xbar2,sd2,xbar3),sd3);
    
    vardstar = [ddstarmu1 ddstarsd1 ddstarmu2 ddstarsd2 ddstarmu3 ddstarsd3]*V*...
                    [ddstarmu1 ddstarsd1 ddstarmu2 ddstarsd2 ddstarmu3 ddstarsd3]';
   
    dstarl = dstar - 1.96*sqrt(vardstar);
    dstaru = dstar + 1.96*sqrt(vardstar);
    
    
    dc1dsmu1 = derivest(@(xbar) c1starfunmu1(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar1);
    dc1dsmu2 = derivest(@(xbar) c1starfunmu2(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar2);
    dc1dsmu3 = derivest(@(xbar) c1starfunmu3(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar3);
    
    dc1dssd1 = derivest(@(sd) c1starfunsd1(sd,xbar1,xbar2,sd2,xbar3,sd3),sd1);
    dc1dssd2 = derivest(@(sd) c1starfunsd2(sd,xbar1,sd1,xbar2,xbar3,sd3),sd2);
    dc1dssd3 = derivest(@(sd) c1starfunsd3(sd,xbar1,sd1,xbar2,sd2,xbar3),sd3);
    
    dc2dsmu1 = derivest(@(xbar) c2starfunmu1(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar1);
    dc2dsmu2 = derivest(@(xbar) c2starfunmu2(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar2);
    dc2dsmu3 = derivest(@(xbar) c2starfunmu3(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar3);
    
    dc2dssd1 = derivest(@(sd) c2starfunsd1(sd,xbar1,xbar2,sd2,xbar3,sd3),sd1);
    dc2dssd2 = derivest(@(sd) c2starfunsd2(sd,xbar1,sd1,xbar2,xbar3,sd3),sd2);
    dc2dssd3 = derivest(@(sd) c2starfunsd3(sd,xbar1,sd1,xbar2,sd2,xbar3),sd3);
    
    
    c1vards = [dc1dsmu1 dc1dssd1 dc1dsmu2 dc1dssd2 dc1dsmu3 dc1dssd3]*V* ...
    [dc1dsmu1 dc1dssd1 dc1dsmu2 dc1dssd2 dc1dsmu3 dc1dssd3]';

    c2vards = [dc2dsmu1 dc2dssd1 dc2dsmu2 dc2dssd2 dc2dsmu3 dc2dssd3]*V* ...
    [dc2dsmu1 dc2dssd1 dc2dsmu2 dc2dssd2 dc2dsmu3 dc2dssd3]';
             

    c1dsl = c1dstar - 1.96*sqrt(c1vards);
    c1dsu = c1dstar + 1.96*sqrt(c1vards);
    
    c2dsl = c1dstar - 1.96*sqrt(c1vards);
    c2dsu = c1dstar + 1.96*sqrt(c1vards);
    
    dstarout = [dstar dstarl dstaru];
    c1out = [c1dstar c1dsl c1dsu];
    c2out = [c2dstar c2dsl c2dsu];
    tcrout = [TCR1ds TCR2ds TCR3ds];
    %{
    out = {strcat('D* (CI) = ',string(dstar),' (',string(dstarl),', ',string(dstaru),')');
               strcat('c_{1} (CI) = ',string(c1dstar),' (',string(c1dsl),', ',string(c1dsu),')');
               strcat('c_{2} (CI) = ',string(c2dstar),' (',string(c2dsl),', ',string(c2dsu),')');
               strcat('var(c_{1}) = ',string(c1vards));
               strcat('var(c_{2}) = ',string(c2vards));
               strcat('(TCR_{1}, TCR_{2}, TCR_{3}) = ','(',string(TCR1ds),', ',string(TCR2ds),', ',string(TCR3ds),')')};
    %}
        switch plotsurf
            case 'yes'
                ROCsurf=@(p1,p3) (norminv(1-p3,xbar3,sd3)>norminv(p1,xbar1,sd1)).*(normcdf(norminv(1-p3,xbar3,sd3),xbar2,sd2)-normcdf(norminv(p1,xbar1,sd1),xbar2,sd2));
                [p1,p3] = meshgrid(0:0.01:1, 0:0.01:1);
                ROCsparam=(normcdf(norminv(1-p3,xbar3,sd3),xbar2,sd2)-normcdf(norminv(p1,xbar1,sd1),xbar2,sd2));
    
                figure
                surf(p1,p3,ROCsparam) %plot the ROC surface
    
                hold on
                xlim([0 1])
                ylim([0 1])
                zlim([0 1])
                line([1 TCR1ds], [1 TCR3ds], [1 TCR2ds],'Color','r','LineWidth',1.5)
                shading interp
                alpha 0.6
                xlabel('TCR1')
                ylabel('TCR3')
                zlabel('TCR2')
                set(gcf,'color','w');
                hold off
        end
        
        
    
%=========================================================%
% Assume you can transform to normality - use bootstrap   %
% that takes into account the variability of lambda       %
%=========================================================%
elseif method == 'boxcox'
    
    x1bc = x1;
    x2bc = x2;
    x3bc = x3;
    
    if isrow(x1bc)
       x1bc = x1bc';
    end
    
    if isrow(x2bc)
        x2bc = x2bc';
    end
    
    if isrow(x3bc)
        x3bc = x3bc';
    end
    
    x = [x1bc; x2bc; x3bc];
    if (min(x) < 0)
    shift = abs(min(x)) + 1;

    x1bc = x1bc + shift;
    x2bc = x2bc + shift;
    x3bc = x3bc + shift;
    end
    
    [x1t, x2t, x3t, lam] = boxcox3d(x1bc,x2bc,x3bc);

    outc = COVlam3d(x1bc,x2bc,x3bc);
    
    V = outc(1:6,1:6);
    
    xbar1 = mean(x1t);
    xbar2 = mean(x2t);
    xbar3 = mean(x3t);
    
    sd1 = std(x1t)*sqrt((length(x1t)-1)/length(x1t));
    sd2 = std(x2t)*sqrt((length(x2t)-1)/length(x2t));
    sd3 = std(x3t)*sqrt((length(x3t)-1)/length(x3t));
   
    dist = @(c)(norminv(1 - (1 - normcdf(c(2),xbar3,sd3)),xbar3,sd3) > norminv(normcdf(c(1),xbar1,sd1),xbar1,sd1))*...
               (sqrt((1 - normcdf(c(1),xbar1,sd1))^2 ...
                   + (1 - (normcdf(c(2),xbar2,sd2) - normcdf(c(1),xbar2,sd2)))^2 + ...
               (normcdf(c(2),xbar3,sd3))^2)); %function to be minimized

    c0 = [(xbar1+xbar2)/2 (xbar2+xbar3)/2];
    options = optimset('MaxIter',20000,'MaxFunEvals',20000,'TolX',10e-15);
    [cstar, dstar]= fminsearch(dist,c0,options);
    c1star= cstar(1);
    c2star= cstar(2);
    
    TCR1ds = normcdf((c1star-xbar1)/sd1);
    TCR2ds = normcdf((c2star-xbar2)/sd2) - normcdf((c1star-xbar2)/sd2);
    TCR3ds = normcdf((xbar3-c2star)/sd3);
    
    ddstarmu1 = derivest(@(xbar)dstarfunmu1(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar1);
    ddstarmu2 = derivest(@(xbar)dstarfunmu2(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar2);
    ddstarmu3 = derivest(@(xbar)dstarfunmu3(xbar,xbar1,sd1,xbar2,sd2,xbar3,sd3),xbar3);

    ddstarsd1 = derivest(@(sd)dstarfunsd1(sd,xbar1,xbar2,sd2,xbar3,sd3),sd1);
    ddstarsd2 = derivest(@(sd)dstarfunsd2(sd,xbar1,sd1,xbar2,xbar3,sd3),sd2);
    ddstarsd3 = derivest(@(sd)dstarfunsd3(sd,xbar1,sd1,xbar2,sd2,xbar3),sd3);
    
    vardstar = [ddstarmu1 ddstarsd1 ddstarmu2 ddstarsd2 ddstarmu3 ddstarsd3]*V*...
               [ddstarmu1 ddstarsd1 ddstarmu2 ddstarsd2 ddstarmu3 ddstarsd3]';
           
    dstarl = dstar - 1.96*sqrt(vardstar);
    dstaru = dstar + 1.96*sqrt(vardstar);
    
    if (min(x) < 0)
    c1dstar = (c1star*lam+1)^(1/lam) - shift;
    c2dstar = (c2star*lam+1)^(1/lam) - shift;
    
    else
        c1dstar = (c1star*lam+1)^(1/lam);
        c2dstar = (c2star*lam+1)^(1/lam);
    end
    
    ROCsurf=@(p1,p3) (norminv(1-p3,xbar3,sd3)>norminv(p1,xbar1,sd1)).*(normcdf(norminv(1-p3,xbar3,sd3),xbar2,sd2)-normcdf(norminv(p1,xbar1,sd1),xbar2,sd2));
    
    x1o = x1;
    x2o = x2;
    x3o = x3;
        
    boots = 1;
    
    while boots <= 400
    atx1 = randsample(1:length(x1o),length(x1o),1);
    atx2 = randsample(1:length(x2o),length(x2o),1);
    atx3 = randsample(1:length(x3o),length(x3o),1);
    
    x1bs = x1o(atx1);
    x2bs = x2o(atx2);
    x3bs = x3o(atx3);
    shift = 0;
    
    if (min(x1bs) < 0) || (min(x2bs) < 0) || (min(x3bs) < 0)
    
    shift = max([-min(x1o) -min(x2o) -min(x3o)]) + 1;
    
    x1bs = x1bs + shift;
    x2bs = x2bs + shift;
    x3bs = x3bs + shift;
    end
    
    [x1b, x2b, x3b, lamb] = boxcox3d(x1bs,x2bs,x3bs);
    
    xbar1b = mean(x1b);
    xbar2b = mean(x2b);
    xbar3b = mean(x3b);
    
    sd1b = std(x1b)*sqrt((length(x1b)-1)/length(x1b));
    sd2b = std(x2b)*sqrt((length(x2b)-1)/length(x2b));
    sd3b = std(x3b)*sqrt((length(x3b)-1)/length(x3b));
    
    
    distbs = @(c)(norminv(1 - (1 - normcdf(c(2),xbar3b,sd3b)),xbar3b,sd3b) > norminv(normcdf(c(1),xbar1b,...
            sd1b),xbar1b,sd1b))*(sqrt((1 - normcdf(c(1),xbar1b,sd1b))^2 + (1 - (normcdf(c(2),xbar2b,sd2b)...
            - normcdf(c(1),xbar2b,sd2b)))^2 + ...
           (normcdf(c(2),xbar3b,sd3b))^2)); %function to be minimized

    c0 = [(xbar1b+xbar2b)/2 (xbar2b+xbar3b)/2];
    options = optimset('MaxIter',20000,'MaxFunEvals',20000,'TolX',10e-15);
    [cstarbs, ~]= fminsearch(distbs,c0,options);
    
    c1_starbs= cstarbs(1);
    c2_starbs= cstarbs(2);
    
    c1starbs(boots) = (c1_starbs*lamb+1)^(1/lamb) - shift;
    c2starbs(boots) = (c2_starbs*lamb+1)^(1/lamb) - shift;
    
    boots = boots + 1;
    
    end
    
    c1vards = var(c1starbs);
    c2vards = var(c2starbs);
    
    
    c1dsl = c1dstar - 1.96*sqrt(c1vards);
    c1dsu = c1dstar + 1.96*sqrt(c1vards);
    c2dsl = c2dstar - 1.96*sqrt(c2vards);
    c2dsu = c2dstar + 1.96*sqrt(c2vards);
    
    dstarout = [dstar dstarl dstaru];
    c1out = [c1dstar c1dsl c1dsu];
    c2out = [c2dstar c2dsl c2dsu];
    tcrout = [TCR1ds TCR2ds TCR3ds];

        switch plotsurf
            case 'yes'
                [p1,p3] = meshgrid(0:0.01:1, 0:0.01:1);
                ROCsparam=(normcdf(norminv(1-p3,xbar3,sd3),xbar2,sd2)-normcdf(norminv(p1,xbar1,sd1),xbar2,sd2));
    
                figure
                surf(p1,p3,ROCsparam) %plot the ROC surface
                hold on

                xlim([0 1])
                ylim([0 1])
                zlim([0 1])
                line([1 TCR1ds], [1 TCR3ds], [1 TCR2ds],'Color','r','LineWidth',1.5)
                shading interp
                alpha 0.6
                xlabel('TCR1')
                ylabel('TCR3')
                zlabel('TCR2')
                set(gcf,'color','w');
                hold off
        end
    
elseif method == 'kernel'
    bw1 = 0.9*min([std(x1) iqr(x1)/1.34])*length(x1)^(-0.2);
    bw2 = 0.9*min([std(x2) iqr(x2)/1.34])*length(x2)^(-0.2);
    bw3 = 0.9*min([std(x3) iqr(x3)/1.34])*length(x3)^(-0.2);
    
    dist = @(c) sqrt((1 - ksdensity(x1,c(1),'Function','cdf','Bandwidth',bw1))^2 + ... 
                    (1 - (ksdensity(x2,c(2),'Function','cdf','Bandwidth',bw2) - ksdensity(x2,c(1),'Function','cdf','Bandwidth',bw2)))^2 + ...
                    (ksdensity(x3,c(2),'Function','cdf','Bandwidth',bw3))^2);
             
   c0 = [(mean(x1)+mean(x2))/2 (mean(x2)+mean(x3))/2];
   options = optimset('MaxIter',20000,'MaxFunEvals',20000,'TolX',10e-15);
   [cdstar, dstar]= fminsearch(dist,c0,options);
   
   c1dstar = cdstar(1);
   c2dstar = cdstar(2);
    
    boots = 1;
    while boots <= 400
      boots
      atx1 = randsample(1:length(x1),length(x1),1);
      atx2 = randsample(1:length(x2),length(x2),1);
      atx3 = randsample(1:length(x3),length(x3),1);
    
      x1b = x1(atx1);
      x2b = x2(atx2);
      x3b = x3(atx3);
   
      xbar1bs = mean(x1b);
      xbar2bs = mean(x2b);
      xbar3bs = mean(x3b);
      
      bwbs1 = 0.9*min([std(x1b) iqr(x1b)/1.34])*length(x1)^(-0.2);
      bwbs2 = 0.9*min([std(x2b) iqr(x2b)/1.34])*length(x2)^(-0.2);
      bwbs3 = 0.9*min([std(x3b) iqr(x3b)/1.34])*length(x3)^(-0.2);
      
      distbs = @(c) sqrt((1 - ksdensity(x1b,c(1),'Function','cdf','Bandwidth',bwbs1))^2 + ... 
                    (1 - (ksdensity(x2b,c(2),'Function','cdf','Bandwidth',bwbs2) - ksdensity(x2b,c(1),'Function','cdf','Bandwidth',bwbs2)))^2 + ...
                    (ksdensity(x3b,c(2),'Function','cdf','Bandwidth',bwbs3))^2);
             
      c0 = [(mean(x1b)+mean(x2b))/2 (mean(x2b)+mean(x3b))/2];
      [~, dstarbs(boots)]= fminsearch(distbs,c0,options);
      
      eps1 = normrnd(0,bw1,1,length(x1));
      eps2 = normrnd(0,bw2,1,length(x2));
      eps3 = normrnd(0,bw3,1,length(x3));
      
      x1be = x1b + eps1;
      x2be = x2b + eps2;
      x3be = x3b + eps3;
      
      distbse = @(c) sqrt((1 - ksdensity(x1b,c(1),'Function','cdf','Bandwidth',bwbs1))^2 + ... 
                    (1 - (ksdensity(x2b,c(2),'Function','cdf','Bandwidth',bwbs2) - ksdensity(x2b,c(1),'Function','cdf','Bandwidth',bwbs2)))^2 + ...
                    (ksdensity(x3b,c(2),'Function','cdf','Bandwidth',bwbs3))^2);
                
      cdstar = fminsearch(distbse,c0,options);
      c1dstarbs(boots) = cdstar(1);
      c2dstarbs(boots) = cdstar(2);
      
      tcr1dstarbs(boots) = ksdensity(x1b,c1dstarbs(boots),'Function','cdf','Bandwidth',bwbs1);
      tcr2dstarbs(boots) = ksdensity(x2b,c2dstarbs(boots),'Function','cdf','Bandwidth',bwbs2) - ksdensity(x2b,c1dstarbs(boots),'Function','cdf','Bandwidth',bwbs2);
      tcr3dstarbs(boots) = 1 - ksdensity(x3b,c2dstarbs(boots),'Function','cdf','Bandwidth',bwbs3);
      
      if c1dstarbs(boots) > c2dstarbs(boots)
          aa = aa + 1;
          continue
      end
      boots = boots + 1;
    end
    
    dstarvar = var(dstarbs);
    dstarl = dstar - 1.96*sqrt(dstarvar);
    dstaru = dstar + 1.96*sqrt(dstarvar);
    
    c1vards = var(c1dstarbs);
    c2vards = var(c2dstarbs);

    c1dsl = c1dstar - 1.96*sqrt(c1vards);
    c1dsu = c1dstar + 1.96*sqrt(c1vards);
    c2dsl = c2dstar - 1.96*sqrt(c2vards);
    c2dsu = c2dstar + 1.96*sqrt(c2vards);
    
    TCR1ds = ksdensity(x1,c1dstar,'Function','cdf','Bandwidth',bw1);
    TCR2ds = ksdensity(x2,c2dstar,'Function','cdf','Bandwidth',bw2) - ksdensity(x2,c1dstar,'Function','cdf','Bandwidth',bw2);
    TCR3ds = 1 - ksdensity(x3,c2dstar,'Function','cdf','Bandwidth',bw3);
    
    dstarout = [dstar dstarl dstaru];
    c1out = [c1dstar c1dsl c1dsu];
    c2out = [c2dstar c2dsl c2dsu];
    tcrout = [TCR1ds TCR2ds TCR3ds];
    
        switch plotsurf
            case 'yes'      
               gr1 = 0:.01:1;
               gr2 = 0:.01:1;
               [pp1,pp3] = meshgrid(gr1, gr2);
               p1=pp1(1:end)';
               p3=pp3(1:end)';

                for i = 1:length(p1)
                        ROCsurf(i) = (ksdensity(x3,1-p3(i),'Function','icdf','Bandwidth',bw3)>ksdensity(x1,p1(i),'Function','icdf','Bandwidth',bw1))*(ksdensity(x2,ksdensity(x3,1-p3(i),'Function','icdf','Bandwidth',bw3),'Function','cdf','Bandwidth',bw2) - ...
                         ksdensity(x2,ksdensity(x1,p1(i),'Function','icdf','Bandwidth',bw1),'Function','cdf','Bandwidth',bw2));
                end
    
                p1=reshape(p1,size(pp1,1),size(pp1,2));
                p3=reshape(p3,size(pp3,1),size(pp3,2));
    
                ROCsurf=reshape(ROCsurf,size(pp1,1),size(pp1,2));
    
                ROCsparam = @(p1,p3) (ksdensity(x3,1-p3,'Function','icdf','Bandwidth',bw3)>ksdensity(x1,p1,'Function','icdf','Bandwidth',bw1)).*(ksdensity(x2,ksdensity(x3,1-p3,'Function','icdf','Bandwidth',bw3),'Function','cdf','Bandwidth',bw2) - ...
                                     ksdensity(x2,ksdensity(x1,p1,'Function','icdf','Bandwidth',bw1),'Function','cdf','Bandwidth',bw2));
   
    
                figure
                surf(p1,p3,ROCsurf)
                hold on
                xlim([0 1])
                ylim([0 1])
                zlim([0 1])
                line([1 TCR1ds], [1 TCR3ds], [1 TCR2ds],'Color','r','LineWidth',1.5)
                shading interp
                alpha 0.6
                xlabel('TCR1')
                ylabel('TCR3')
                zlabel('TCR2')
                set(gcf,'color','w');
                hold off
        end
end
end
